*DECK THMTRS
      SUBROUTINE THMTRS(IMPX,IC,JC,KC,LC,XBURN,HX,HY,HZ,DTIME,
     > CFLUX,POROS,NFU,NFD,NDTOT,SASS,FCOOL,FFUEL,ACOOL,HD,PCH,RGG,RIG,
     > RC,MAXITC,MAXIT1,MAXITL,ERMAXT,SPDIN,TINLET,PINLET,FRACPU,
     > ICONDF,NCONDF,KCONDF,UCONDF,ICONDC,NCONDC,KCONDC,UCONDC,IHGAP,
     > KHGAP,IHCONV,KHCONV,IFRCDI,POW,MFLXT0,RHOT0,PREST0,ENTHT0,
     > TEMPT0,TCENT0,TCOMB,DCOOL,TCOOL,TSURF,MFLXT,RHOT,PREST,ENTHT,
     > TEMPT,TCENTT,RTIME)
*
*-----------------------------------------------------------------------
*
*Purpose:
* driver of the transient thermal-hydraulics module for a single time
* iteration
*
*Copyright:
* Copyright (C) 2013 Ecole Polytechnique de Montreal.
*
*Author(s): P. Gallet
*
*Parameters: input
* IMPX    printing index (=0 for no print).
* IC      number of meshes along X direction.
* JC      number of meshes along Y direction.
* KC      number of meshes along Z direction (channel direction).
* LC      number of bundles along the axial (Z) direction.
* XBURN   burnup distribution in MWday/tonne.
* HX      X-directed mesh widths.
* HY      Y-directed mesh widths.
* HZ      Z-directed mesh widths.
* DTIME   time step in s.
* CFLUX   critical heat flux in W/m^2.
* POROS   oxyde porosity.
* NFU     number of active fuel rods in the fuel bundle.
* NFD     number of discretisation points in fuel regions.
* NDTOT   number of total discretization points in the the fuel
*         pellet and the cladding.
* SASS    assembly surface in m^2.
* FCOOL   power density fraction in coolant.
* FFUEL   power density fraction in fuel.
* ACOOL   assembly coolant cross section in m^2.
* HD      hydraulic diameter of one assembly in m.
* PCH     heating perimeter in m.
* RGG     external clad rod radius in m.
* RIG     internal clad rod radius in m.
* RC      fuel pellet radius in m.
* MAXITC  maximum number of convectin iterations.
* MAXIT1  maximum number of conduction iterations.
* MAXITL  maximum number of center-pellet iterations.
* ERMAXT  convergence criterion.
* SPDIN   inlet flow velocity in m/s.
* TINLET  inlet temperature in K.
* PINLET  inlet pressure in Pa.
* FRACPU  plutonium fraction in fuel.
* ICONDF  fuel conductivity flag (0=Stora-Chenebault or COMETHE/
*         1=user-provided polynomial + inverse term).
* NCONDF  degree of user-provided fuel conductivity polynomial.
* KCONDF  polynomial coefficients for fuel conductivity in W/m/K^(k+1)
*         (except for the two last coefficients which belongs to the
*         inverse term).
* UCONDF  required unit of temperature in polynomial for fuel
*         conductivity (KELVIN or CELSIUS).
* ICONDC  clad conductivity flag (0=default/1=user-provided
*         polynomial).
* NCONDC  degree of user-provided clad conductivity polynomial.
* KCONDC  polynomial coefficients for clad conductivity in W/m/K^(k+1).
* UCONDC  required unit of temperature in polynomial for clad
*         conductivity (KELVIN or CELSIUS).
* IHGAP   flag indicating HGAP chosen (0=default/1=user-provided).
* KHGAP   fixed user-provided HGAP value in W/m^2/K.
* IHCONV  flag indicating HCONV chosen (0=default/1=user-provided).
* KHCONV  fixed user-provided HCONV value in W/m^2/K.
* IFRCDI  flag indicating if average approximation is forced during
*         fuel conductivity evaluation (0=default/1=average
*         approximation forced).
* POW     power distribution in W.
* MFLXT0  coolant mass flux distribution in the reactor at t-1 in kg/m^2/s.
* RHOT0   density distribution in the reactor at t-1 in kg/m3.
* PREST0  pressure distribution in the reactor at t-1 in Pa.
* ENTHT0  enthalpy distribution in the reactor at t-1 in J/kg.
* TEMPT0  temperature distribution in the fuel rod at t-1 in K.
* TCENT0  center fuel pellet temperature distribution at t-1 in K.
* RTIME   time in s.
*
*Parameters: output
* TCOMB   averaged fuel temperature distribution in K.
* DCOOL   averaged coolant density distribution in g/cc.
* TCOOL   averaged coolant temperature distribution in K.
* TSURF   surface fuel temperature distribution in K.
* MFLXT   coolant mass flux distribution in the reactor at t in kg/m^2/s.
* RHOT    coolant density distribution in the reactor at t in kg/m^3.
* PREST   coolant pressure distribution in the reactor at t in Pa.
* ENTHT   coolant enthalpy distribution in the reactor at t in J/kg.
* TEMPT   temperature distribution in the fuel rod at t in K.
* TCENTT  center fuel pellet temperature distribution at t in K.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IMPX,IC,JC,KC,LC,NFU,NFD,NDTOT,MAXITC,MAXIT1,MAXITL
      REAL XBURN(IC,JC,KC),HX(IC),HY(JC),HZ(KC),DTIME,CFLUX,POROS,SASS,
     > FCOOL,FFUEL,ACOOL,HD,PCH,RGG,RIG,RC,ERMAXT,SPDIN,TINLET,PINLET,
     > FRACPU,KCONDF(NCONDF+3),KCONDC(NCONDC+1),KHGAP,KHCONV,
     > POW(IC,JC,KC),MFLXT0(IC,JC,KC),RHOT0(IC,JC,KC),PREST0(IC,JC,KC),
     > ENTHT0(IC,JC,KC),TEMPT0(IC,JC,KC,NDTOT),TCOMB(IC,JC,KC),
     > DCOOL(IC,JC,KC),TCOOL(IC,JC,KC),TSURF(IC,JC,KC),RHOT(IC,JC,KC),
     > PREST(IC,JC,KC),TCLAD(IC,JC,KC),TCENTT(IC,JC,KC),
     > ENTHT(IC,JC,KC),TEMPT(IC,JC,KC,NDTOT),MFLXT(IC,JC,KC),
     > TCENT0(IC,JC,KC),RTIME
      CHARACTER UCONDF*12,UCONDC*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER(KMAXO=100,MAXNPO=20)
      PARAMETER(ZMCM=1.0E2,ZKILO=1.0E3,PES=9.81)
*     compute the actual length of each channel without reflector
*     regions, NR = number of reflector regions in the axial direction
      INTEGER NR,NHR
      REAL TRE11(MAXNPO),ENT(4),FRO(MAXNPO),MFLXIN,
     > RHOIN,HINLET,MUIN,DV(IC,JC,KC),DZ,F,PARAM1,PARAM2,PARAM3,LZ,
     > ERRG,ERRP,ERRH,ERR,DELTH,HINI,AG(LC,LC+1),
     > AP(LC,LC+1),AH(LC,LC+1),POWLIN(IC,JC,KC),PHI(IC,JC,KC),
     > RET(IC,JC,LC),MUT(IC,JC,KC),HGAPM(IC,JC,KC),RESG(IC,JC,KC),
     > RESP(IC,JC,KC),RESH(IC,JC,KC),QFUEL(IC,JC,KC),QCOOL(IC,JC,KC),
     > ERMAX,TC0,ERRR,RHOT1(IC,JC,KC),HCONV(IC,JC,KC),PARAM11,PARAM12,
     > PARAM13,PARAM21,PARAM22,PARAM23,XC(IC,JC,KC),XCI(IC,JC,KC),ERRX,
     > TSAT,DSAT
      CHARACTER HSMG*131
      REAL XS(4)
      DATA XS/-0.861136,-0.339981,0.339981,0.861136/
*----
*  CALCUlATE THE NUMBER OF REFLECTOR REGIONS IN THE AXIAL DIRECTION
*----
      NR=KC-LC
      NHR=NR/2
*----
*  COMPUTE THE INLET FLOW ENTHALPY AND VELOCITY
*----
         CALL THMSAT(PINLET,TSAT)
         IF(TINLET.GT.TSAT) THEN
            WRITE(HSMG,'(27HTHMTRS: INLET TEMPERATURE (,1P,E12.4,
     1      40H K) GREATER THAN SATURATION TEMPERATURE.)') TINLET
            CALL XABORT(HSMG)
         ENDIF
         CALL THMPT(PINLET,TINLET,RHOIN,HINLET,R3,MUIN,R5)
         MFLXIN=SPDIN*RHOIN
         IF(RTIME.LE.20.0) THEN
            MFLXIN=MFLXIN !-MFLXIN*0.10/60.0*RTIME
         ELSEIF(RTIME.LE.80.0) THEN
            MFLXIN=MFLXIN  !-MFLXIN*0.10/60.0*(RTIME-20.0)
         ELSE 
            MFLXIN=MFLXIN !*0.90
         ENDIF
*----
*  LOOP OVER REACTOR CHANNELS
*----
      IF(NDTOT.GT.MAXNPO) CALL XABORT('THMTRS: MAXNPO OVERFLOW')
      DO I=1,IC
      DO J=1,JC
       IF(POW(I,J,(KC+1)/2).GT.0.0) THEN
*----
*  MAIN LOOP ALONG THE 1D CHANNEL.
*----
         DO K=1,LC
*----
*  COMPUTE THE LINEAR POWER, THE VOLUMIC POWER, THE THERMAL EXCHANGE
*  COEFFICIENT OF THE GAP AND THE THERMAL HEAT FLUX IN EACH ALONG THE
*  CHANNEL
*----
           DV(I,J,K+NHR)=HX(I)*HY(J)*HZ(K+NHR)
*          linear power in W/m
           POWLIN(I,J,K+NHR)=(POW(I,J,K+NHR)/DV(I,J,K+NHR))*SASS/NFU
*          volumic power in W/m^3
           QFUEL(I,J,K+NHR)=POW(I,J,K+NHR)*FFUEL/DV(I,J,K+NHR)
           QCOOL(I,J,K+NHR)=POW(I,J,K+NHR)*FCOOL/DV(I,J,K+NHR)
*          thermal exchange coefficient of the gap in W/m^2/K.
           IF(IHGAP.EQ.0) THEN
              CALL THMGAP(POWLIN(I,J,K+NHR),XBURN(I,J,K+NHR),
     >         HGAPM(I,J,K+NHR))
           ELSE IF(IHGAP.EQ.1) THEN
              HGAPM(I,J,K+NHR)=KHGAP
           ENDIF
*          thermal heat flux in W/m^2
           PHI(I,J,K+NHR)=0.5*QFUEL(I,J,K+NHR)*RC*RC/RGG
           IF(PHI(I,J,K+NHR).GT.CFLUX) THEN
              WRITE(HSMG,'(23HTHMTRS: THE HEAT FLUX (,1P,E12.4,5H) IS,
     >        37HGREATER THAN THE CRITICAL HEAT FLUX (,E12.4,2H).)')
     >        PHI(I,J,K+NHR),CFLUX
              CALL XABORT(HSMG)
           ENDIF
*----
*  INITIALIZATION OF THE THERMO-HYDRAULICAL PROPERTIES OF THE FLUID
*----
           RHOT(I,J,K+NHR)=RHOT0(I,J,K+NHR)
           MUT(I,J,K+NHR)=MUIN
           MFLXT(I,J,K+NHR)=MFLXT0(I,J,K+NHR)
           PREST(I,J,K+NHR)=PREST0(I,J,K+NHR)
           ENTHT(I,J,K+NHR)=ENTHT0(I,J,K+NHR)
           XC(I,J,K+NHR)=0.0
           XCI(I,J,K+NHR)=XC(I,J,K+NHR)
         ENDDO
       ENDIF
      ENDDO
      ENDDO
*----
*  ITERATIVE PROCEDURE
*----
      DO I=1,IC
      DO J=1,JC
         IF(POW(I,J,(KC+1)/2).GT.0.0) THEN
            ITERC=0
   10       ITERC=ITERC+1
            IF(ITERC.GT.MAXITC) THEN
               CALL XABORT('THMTRS: CONVERGENCE FAILURE IN
     >         CONVECTION CALCULATION')
            ENDIF
*          reinitialization of all the matrixes
           DO K=1,LC
           DO K1=1,(LC+1)
              AG(K,K1)=0.0
              AP(K,K1)=0.0
              AH(K,K1)=0.0
           ENDDO
           ENDDO
           DO K=2,(LC-1)
              DZ=(HZ(K-1+NHR)+HZ(K+1+NHR)+2.0*HZ(K+NHR))/2.0
              AG(K,K-1)=-1.0
              AG(K,K+1)=1.0
              AG(K,LC+1)=(RHOT0(I,J,K+NHR)-RHOT(I,J,K+NHR))/DTIME*DZ
           ENDDO
           DZ=HZ(1+NHR)+HZ(2+NHR)
           AG(1,2)=1.0
           AG(1,LC+1)=MFLXIN+(RHOT0(I,J,1+NHR)-RHOT(I,J,1+NHR))/DTIME
     >     *DZ
           AG(LC,LC-2)=1.0
           AG(LC,LC-1)=-4.0
           AG(LC,LC)=3.0
           DZ=(HZ(LC-2+NHR)+2.0*HZ(LC-1+NHR)+HZ(LC+NHR))/2.0
           AG(LC,LC+1)=(RHOT0(I,J,LC+NHR)-RHOT(I,J,LC+NHR))/DTIME*DZ
           CALL ALSB(LC,1,AG,IER,LC)
           IF(IER.NE.0) CALL XABORT('THMTRS: AG SINGULAR MATRIX.')
*----
*  MOMENTUM VECTOR CALCULATION WITH THE MOMENTUM CONSERVATION EQUATION
*----
           DO K=1,LC
              RET(I,J,K)=AG(K,LC+1)*HD/MUT(I,J,K+NHR)
           ENDDO
           DO K=2,(LC-1)
              AP(K,K-1)=-1.0
              AP(K,K+1)=+1.0
              DZ=HZ(K+NHR)+0.5*(HZ(K+1+NHR)+HZ(K-1+NHR))
              PARAM1=(AG(K,LC+1)-MFLXT0(I,J,K+NHR))/DTIME*DZ
              PARAM21=AG(K+1,LC+1)**2.0/RHOT(I,J,K+1+NHR)
              PARAM22=AG(K-1,LC+1)**2.0/RHOT(I,J,K-1+NHR)
              PARAM2=PARAM21-PARAM22
              CALL THMFRI(RET(I,J,K),F)
              PARAM31=RHOT(I,J,K+NHR)*PES
              PARAM32=0.5*F*AG(K,LC+1)**2.0/HD/RHOT(I,J,K+NHR)
              PARAM3=(PARAM31+PARAM32)*DZ
              AP(K,LC+1)=-PARAM1-PARAM2-PARAM3
           ENDDO
           AP(1,2)=1.0
           DZ=HZ(1+NHR)+HZ(2+NHR)
           PARAM1=(AG(1,LC+1)-MFLXT0(I,J,1+NHR))/DTIME*DZ
           PARAM21=AG(2,LC+1)**2.0/RHOT(I,J,2+NHR)
           PARAM22=MFLXIN**2.0/RHOIN+PINLET
           PARAM2=PARAM21-PARAM22
           CALL THMFRI(RET(I,J,1),F)
           PARAM31=RHOT(I,J,1+NHR)*PES
           PARAM32=0.5*F*AG(1,LC+1)**2.0/HD/RHOT(I,J,1+NHR)
           PARAM3=(PARAM31+PARAM32)*DZ
           AP(1,LC+1)=-PARAM1-PARAM2-PARAM3
           AP(LC,LC-2)=1.0
           AP(LC,LC-1)=-4.0
           AP(LC,LC)=3.0
           DZ=HZ(LC-1+NHR)+0.5*(HZ(LC+NHR)+HZ(LC-2+NHR))
           PARAM1=(AG(LC,LC+1)-MFLXT0(I,J,LC+NHR))/DTIME*DZ
           PARAM21=AG(LC,LC+1)**2.0/RHOT(I,J,LC+NHR)
           PARAM22=AG(LC-1,LC+1)**2.0/RHOT(I,J,LC+NHR-1)
           PARAM23=AG(LC-2,LC+1)**2.0/RHOT(I,J,LC+NHR-2)
           PARAM2=3.0*PARAM21-4.0*PARAM22+1.0*PARAM23
           CALL THMFRI(RET(I,J,LC),F)
           PARAM31=RHOT(I,J,LC+NHR)*PES
           PARAM32=0.5*F*AG(LC,LC+1)**2.0/HD/RHOT(I,J,LC+NHR)
           PARAM3=(PARAM31+PARAM32)*DZ
           AP(LC,LC+1)=-PARAM1-PARAM2-PARAM3
           CALL ALSB(LC,1,AP,IER,LC)
           IF(IER.NE.0) CALL XABORT('THMTRS: AP SINGULAR MATRIX')
*----
*  ENTHALPY VECTOR CALCULATION WITH THE ENERGY CONSERVATION EQUATION
*----
           DO K=2,(LC-1)
              LZ=0.0
              DO I2=1,K
                 LZ=LZ+HZ(I2-1+NHR)
              ENDDO
              DZ=HZ(K+NHR)+0.5*(HZ(K+1+NHR)+HZ(K-1+NHR))
              AH(K,K)=RHOT(I,J,K+NHR)/DTIME*DZ
              AH(K,K-1)=-AG(K-1,LC+1)
              AH(K,K+1)=AG(K+1,LC+1)
              PARAM11=AG(K+1,LC+1)**3.0/(2.0*RHOT(I,J,K+1+NHR)**2.0)
     >        +PES*(LZ+0.5*(HZ(K+NHR)+HZ(K+1+NHR)))*AG(K+1,LC+1)
              PARAM12=AG(K-1,LC+1)**3.0/(2.0*RHOT(I,J,K-1+NHR)**2.0)
     >        +PES*(LZ-0.5*(HZ(K+NHR)+HZ(K-1+NHR)))*AG(K-1,LC+1)
              PARAM1=PARAM11-PARAM12
              PARAM21=AG(K,LC+1)**2.0/(2.0*RHOT(I,J,K+NHR))
     >        -AP(K,LC+1)+RHOT(I,J,K+NHR)*PES*LZ
              PARAM22=MFLXT0(I,J,K+NHR)**2.0/(2.0*RHOT0(I,J,K+NHR))
     >        -PREST0(I,J,K+NHR)+RHOT0(I,J,K+NHR)*PES*LZ
     >        +RHOT0(I,J,K+NHR)*ENTHT0(I,J,K+NHR)
              PARAM2=(PARAM21-PARAM22)*DZ/DTIME
              PARAM3=(PHI(I,J,K+NHR)*(1.0-XCI(I,J,K+NHR))*PCH/ACOOL
     >        +QCOOL(I,J,K+NHR))*DZ 
              AH(K,LC+1)=-PARAM1-PARAM2+PARAM3
           ENDDO
           DZ=HZ(1+NHR)+HZ(2+NHR)
           LZ=0.0
           AH(1,1)=RHOT(I,J,1+NHR)/DTIME*DZ
           AH(1,2)=AG(2,LC+1)
           PARAM11=AG(2,LC+1)**3.0/(2.0*RHOT(I,J,2+NHR)**2.0)+PES*(LZ
     >     +0.5*(HZ(1+NHR)+HZ(2+NHR)))*AG(2,LC+1)
           PARAM12=MFLXIN**3.0/(2.0*RHOIN**2.0)+MFLXIN*HINLET
           PARAM1=PARAM11-PARAM12
           PARAM21=AG(1,LC+1)**2.0/(2.0*RHOT(I,J,1+NHR))-AP(1,LC+1)
     >     +RHOT(I,J,1+NHR)*PES*LZ
           PARAM22=MFLXT0(I,J,1+NHR)**2.0/(2.0*RHOT0(I,J,1+NHR))
     >     -PREST0(I,J,1+NHR)+RHOT0(I,J,1+NHR)*PES*LZ
     >     +RHOT0(I,J,1+NHR)*ENTHT0(I,J,1+NHR)
           PARAM2=(PARAM21-PARAM22)*DZ/DTIME
           PARAM3=(PHI(I,J,1+NHR)*(1.0-XCI(I,J,1+NHR))*PCH/ACOOL
     >     +QCOOL(I,J,1+NHR))*DZ
           AH(1,LC+1)=-PARAM1-PARAM2+PARAM3
           DZ=HZ(LC-1+NHR)+0.5*(HZ(LC+NHR)+HZ(LC-2+NHR))
           DO I2=1,LC
              LZ=LZ+HZ(I2-1+NHR)
           ENDDO
           AH(LC,LC)=RHOT(I,J,LC+NHR)/DTIME*DZ+3.0*AG(LC,LC+1)
           AH(LC,LC-1)=-4.0*AG(LC-1,LC+1)
           AH(LC,LC-2)=1.0*AG(LC-2,LC+1)
           PARAM11=AG(LC,LC+1)**3.0/(2.0*RHOT(I,J,LC+NHR)**2.0)
     >     +PES*LZ*AG(LC,LC+1)
           PARAM12=AG(LC-1,LC+1)**3.0/(2.0*RHOT(I,J,LC-1+NHR)**2.0)
     >     +PES*(LZ-0.5*(HZ(LC+NHR)+HZ(LC-1+NHR)))*AG(LC-1,LC+1)
           PARAM13=AG(LC-2,LC+1)**3.0/(2.0*RHOT(I,J,LC+NHR-2)**2.0)
     >     +PES*(LZ-HZ(LC-1+NHR)-0.5*(HZ(LC+NHR)+HZ(LC+NHR-2)))
     >     *AG(LC-2,LC+1)
           PARAM1=3.0*PARAM11-4.0*PARAM12+1.0*PARAM13
           PARAM21=AG(LC,LC+1)**2.0/(2.0*RHOT(I,J,LC+NHR))
     >     -AP(LC,LC+1)+RHOT(I,J,LC+NHR)*PES*LZ
           PARAM22=MFLXT0(I,J,LC+NHR)**2.0/(2.0*RHOT0(I,J,LC+NHR))
     >     -PREST0(I,J,LC+NHR)+RHOT0(I,J,LC+NHR)*ENTHT0(I,J,LC+NHR)
     >     +RHOT0(I,J,LC+NHR)*PES*LZ
           PARAM2=(PARAM21-PARAM22)*DZ/DTIME
           PARAM3=(PHI(I,J,LC+NHR)*(1.0-XCI(I,J,LC+NHR))*PCH/ACOOL
     >     +QCOOL(I,J,LC+NHR))*DZ 
           AH(LC,LC+1)=-PARAM1-PARAM2+PARAM3
           CALL ALSB(LC,1,AH,IER,LC)
           IF(IER.NE.0) CALL XABORT('THMTRS: AH SINGULAR MATRIX')
           DO K=1,LC
              RESG(I,J,K+NHR)=AG(K,LC+1)
              RESP(I,J,K+NHR)=AP(K,LC+1)
              RESH(I,J,K+NHR)=AH(K,LC+1)
           ENDDO
           DO L=1,NHR
           RESG(I,J,L)=0.0
           RESP(I,J,L)=0.0
           RESH(I,J,L)=0.0
           RESG(I,J,KC-L+1)=0.0
           RESP(I,J,KC-L+1)=0.0
           RESH(I,J,KC-L+1)=0.0
           ENDDO  
*----
*  COMPUTE NEW VALUES OF COOLANT DENSITY
*----
         DO K=1,LC
            CALL THMPH(RESP(I,J,K+NHR),RESH(I,J,K+NHR),R11,T11)
            CALL THMPT(RESP(I,J,K+NHR),T11,RHOT1(I,J,K+NHR),H11,K11,
     >      MUT(I,J,K+NHR),C11)
         ENDDO
*----
*  CONVERGENCE TEST FOR THE ENTHALPY, PRESSURE AND MASS FLUX CALCULATION
*----
*     normally ERMAX=0.00005
      ERMAX=0.0001
      ERRG=0.0
      ERRP=0.0
      ERRH=0.0
      ERRR=0.0
      ERR=0.0
          DO K=1,LC
            ERRG=MAX(ERRG,ABS(MFLXT(I,J,K+NHR)-RESG(I,J,K+NHR))
     >      /MFLXT(I,J,K+NHR))
            ERRP=MAX(ERRP,ABS(PREST(I,J,K+NHR)-RESP(I,J,K+NHR))
     >      /PREST(I,J,K+NHR))
            ERRH=MAX(ERRH,ABS(ENTHT(I,J,K+NHR)-RESH(I,J,K+NHR))
     >      /ENTHT(I,J,K+NHR))
            ERRR=MAX(ERRR,ABS(RHOT(I,J,K+NHR)-RHOT1(I,J,K+NHR))
     >      /RHOT(I,J,K+NHR))
            ERR=MAX(ERRG,ERRP,ERRH,ERRR)
            PREST(I,J,K+NHR)=RESP(I,J,K+NHR)
            MFLXT(I,J,K+NHR)=RESG(I,J,K+NHR)
            ENTHT(I,J,K+NHR)=RESH(I,J,K+NHR)
            RHOT(I,J,K+NHR)=RHOT1(I,J,K+NHR)
          ENDDO
        IF((ERR.LT.ERMAX).AND.(ITERC.NE.1)) THEN
          GO TO 20
        ELSE
          GO TO 10
        ENDIF  
*----
*  CALCULATION OF THE OUTSIDE CLADDING TEMPERATURE AND THE TEMPERATURE
*  DISTRIBUTION IN THE FUEL.
*----
   20   HINI=HINLET
        DO K=1,LC
*----
*  INITIALIZATION OF PINCELL TEMPERATURES
*----
           DO L=1,NDTOT
              TRE11(L)=TEMPT0(I,J,K+NHR,L)
           ENDDO
*----
*  COMPUTE FOUR VALUES OF ENTHALPY IN J/KG TO BE USED IN GAUSSIAN
*  INTEGRATION. DELTH IS THE ENTHALPY INCREASE IN EACH AXIAL MESH.
*----
           IF (K.EQ.1) THEN
               DELTH=ENTHT(I,J,K+NHR)-HINLET
           ELSE
               DELTH=ENTHT(I,J,K+NHR)-ENTHT(I,J,K+NHR-1)
           ENDIF
           DO I1=1,4
              POINT=(1.0+XS(I1))/2.0
              ENT(I1)=HINI+POINT*DELTH
           ENDDO
           HINI=HINI+DELTH
*----
*  TRANSIENT SOLUTION OF THE CONVECTION EQUATIONS ALONG THE 1D
*  CHANNEL.
*----
           CALL THMH2O(PREST(I,J,K+NHR),MFLXT(I,J,K+NHR),ENT,
     >     PHI(I,J,K+NHR),HD,IHCONV,KHCONV,TL,RHO,HCONV(I,J,K+NHR),
     >     TRE11(NDTOT))
*----
*  TRANSIENT SOLUTION OF THE CONDUCTION EQUATIONS IN A FUEL PIN.
*----
           DO K1=1,NFD-1
              FRO(K1)=REAL(K1)
           ENDDO
           RO00=0.0
           DTINV=1.0/DTIME
           ALPHA=1.0
           TC0=TCENT0(I,J,K+NHR)
           CALL THMRES(NFD,NDTOT,MAXIT1,MAXITL,ERMAXT,DTINV,RC,RIG,
     >          RGG,HCONV(I,J,K+NHR),HGAPM(I,J,K+NHR),RO00,TL,FRO,
     >          QFUEL(I,J,K+NHR),XBURN(I,J,K+NHR),POROS,FRACPU,
     >          ICONDF,NCONDF,KCONDF,UCONDF,ICONDC,NCONDC,KCONDC,
     >          UCONDC,IFRCDI,ALPHA,TRE11,TC0)
           DO L=1,NDTOT
              TEMPT(I,J,K+NHR,L)=TRE11(L)
           ENDDO
*----
*  RECOVER MESHWISE TEMPERATURES AND FLUID DENSITY. USE THE ROWLANDS
*  FORMULA TO COMPUTE THE EFFECTIVE FUEL TEMPERATURE.
*----
           TCOMB(I,J,K+NHR)=(4.0*TC0+5.0*TRE11(NFD))/9.0
           TCOOL(I,J,K+NHR)=TL
           TCENTT(I,J,K+NHR)=TC0
           TSURF(I,J,K+NHR)=TRE11(NDTOT)
           TCLAD(I,J,K+NHR)=TRE11(NFD+1)
           DCOOL(I,J,K+NHR)=RHO/ZKILO
        ENDDO
*----
*  CALCULATE THE ERROR ON THE LOCAL BOILING COEFFICIENTS
*----
          ERRX=0.0
          DO K=1,LC
            CALL THMSAT(PREST(I,J,K+NHR),TSAT)
            CALL THMJLO(PREST(I,J,K+NHR),PHI(I,J,K+NHR),
     >      TEMPT(I,J,K+NHR,NDTOT),TCOOL(I,J,K+NHR),TSAT,DSAT,
     >      XC(I,J,K+NHR))
            ERRX=MAX(ERRX,ABS(XC(I,J,K+NHR)-XCI(I,J,K+NHR)))
            XCI(I,J,K+NHR)=XC(I,J,K+NHR)
          ENDDO
          IF((ERRX.LT.ERMAX)) THEN
            GO TO 30
          ELSE
            GO TO 10
          ENDIF
*----
*  PRINT THE OUTLET THERMOHYDRAULICAL PARAMETERS
*----
   30      DO K=1,LC
              RHOT(I,J,K+NHR)=RHOT(I,J,K+NHR)/ZKILO
           ENDDO   
           IF(IMPX.GT.2) THEN
             WRITE(6,'(/16H THMTRS: CHANNEL,2I6/1X,27(1H-))') I,J
             WRITE(6,200) 'TCOMB',(TCOMB(I,J,K),K=1,KC)
             WRITE(6,200) 'TCOOL',(TCOOL(I,J,K),K=1,KC)
             WRITE(6,200) 'TSURF',(TSURF(I,J,K),K=1,KC)
             WRITE(6,200) 'XBOIL',(XC(I,J,K),K=1,KC)
             WRITE(6,200) 'ENTHT',(ENTHT(I,J,K),K=1,KC)
             WRITE(6,200) 'DCOOL',(DCOOL(I,J,K),K=1,KC)
             WRITE(6,200) 'PREST',(PREST(I,J,K),K=1,KC)
             WRITE(6,200) 'MFLXT',(MFLXT(I,J,K),K=1,KC)
             WRITE(6,200) 'TCENT',(TCENTT(I,J,K),K=1,KC)
           ENDIF
        ENDIF
      ENDDO
      ENDDO
      RETURN
  200 FORMAT(/1X,A5,1P,10E12.4/(6X,10E12.4))
      END
